 /*                 JeeNode / JeeNode USB / JeeSMD 
 -------|-----------------------|----|-----------------------|----       
 |       |D3  A1 [Port2]  D5     |    |D3  A1 [port1]  D4     |    |
 |-------|IRQ AIO +3V GND DIO PWR|    |IRQ AIO +3V GND DIO PWR|    |
 | D1|TXD|                                           ---- ----     |
 | A5|SCL|                                       D12|MISO|+3v |    |
 | A4|SDA|   Atmel Atmega 328                    D13|SCK |MOSI|D11 |
 |   |PWR|   JeeNode / JeeNode USB / JeeSMD         |RST |GND |    |
 |   |GND|                                       D8 |BO  |B1  |D9  |
 | D0|RXD|                                           ---- ----     |
 |-------|PWR DIO GND +3V AIO IRQ|    |PWR DIO GND +3V AIO IRQ|    |
 |       |    D6 [Port3]  A2  D3 |    |    D7 [Port4]  A3  D3 |    |
 -------|-----------------------|----|-----------------------|----
 */
 

#include "Sounds.h"
#include <JeeLib.h>


 
//#define DEBUG 
 
int PULSE = 2000;  // how long to sleep inbetween playing, in milliseconds
#define AMP_PWR_PIN A3 // using the Mono Amp breakout board from Sparkfun which has on/off capability: https://www.sparkfun.com/products/11044

const int speakerPin = 3;
const int PayloadLength = 5;

char payload[] = "tweet"; // what we broadcast
char inPayload[PayloadLength];

int payloadLength = PayloadLength; // length of above message

byte needToSend;

// boilerplate for low-power waiting
ISR(WDT_vect) { Sleepy::watchdogEvent(); }

int ListenTime = 5000;
int listenStartTime = 0;

volatile boolean playing = false;
volatile boolean donePlaying = false;

int nextWaitTime = 3000;
int nextWaitTime2 = 4000;

enum Mode { SQUAWK, SLEEP, LISTEN };

Mode mode;


//--------------------------- 
// SETUP --------------------
//---------------------------

void setup () 
{  
    pinMode(A0, INPUT);
    randomSeed(analogRead(A0));
  
    #ifdef DEBUG
    Serial.begin(57600);
    Serial.println("\n[starting]\n");
    delay(1000);
    #endif DEBUG
    
    pinMode(AMP_PWR_PIN, OUTPUT);  // amp on/off pin
    digitalWrite(AMP_PWR_PIN,LOW); // turn off amp

  // turn the radio off in the most power-efficient manner
  //Sleepy::loseSomeTime(32);
  rf12_initialize(1, RF12_868MHZ, 33);
  //rf12_sleep(RF12_SLEEP);
 // rf12_sleep(RF12_WAKEUP);
  // wait another 2s for the power supply to settle
  //Sleepy::loseSomeTime(1000);
  
  startSquawking();
}



void sleep(int sleepTime)
{
  
  #ifdef DEBUG
  Serial.println("sleepy"); 
  delay(1000);
  #endif
 
  mode = SLEEP;
  
  digitalWrite(speakerPin, LOW);
  digitalWrite(AMP_PWR_PIN,LOW);
  delay(40);
  
  //rf12_sleep(RF12_SLEEP);
  /*
  #ifdef DEBUG
  Serial.println(millis());
  delay(500);
  #endif
  */
  
  //Sleepy::loseSomeTime(2000);
  if ( Sleepy::loseSomeTime(sleepTime) )
  {
    
  }
  else
  {
    // something bad happened...
    
    #ifdef DEBUG
    Serial.println("sleep error");
    delay(1000);
    #endif DEBUG
  }
  
  
  //rf12_sleep(RF12_WAKEUP);
  //delay(100);
}


void startListening()
{
    #ifdef DEBUG
    Serial.println("\n[listening]\n");
    delay(250);
    #endif 

   listenStartTime = millis();

    #ifdef DEBUG
    Serial.println(listenStartTime);
    delay(250);
    #endif 

    mode = LISTEN;
}


void startSquawking()
{
  mode = SQUAWK;
 
  #ifdef DEBUG
  Serial.println("PLAY!");
  delay(500);
  #endif 
  
  Sleepy::loseSomeTime(random(200,1200));
  delay(100);
  startPlayback(sample, sizeof(sample));  
}
  

void loop() 
{ 
//  Serial.println("loop");
//  delay(20);
  
  
  if (donePlaying)
  {
    donePlaying = false;
    int sleepyTime = PULSE * random(4,6);
    
     #ifdef DEBUG
     Serial.print("sleepyTime=");
     Serial.println(sleepyTime);
     delay(500);
     #endif DEBUG
     
    // delay(3000);
        
    sleep(sleepyTime);

    playing = false;
    
    startListening();
  }
  
  boolean doneReceiving = rf12_recvDone();
  
  switch (mode)
  {
    case LISTEN:
    {
      int timeDiff = millis() - listenStartTime;
      
      // has time run out?
      if ( timeDiff > ListenTime )
      {
        #ifdef DEBUG
        Serial.print("tdiff:");
        Serial.println(timeDiff);
        delay(1000);
        #endif

        sleep(PULSE * random(1,2));
        
        startSquawking();
      }         // otherwise, listen for calls...
      else if (doneReceiving && rf12_crc == 0 && matchPayload()) 
      {
        #ifdef DEBUG
         Serial.println("RECEIVED");
         delay(250);
        #endif
        
        startSquawking();
      }
    }  
    break;
    
    case SQUAWK:
      // we're playing a sound - broadcast call
      if (rf12_canSend()) 
      {
        /*
           #ifdef DEBUG
           Serial.println("sending ");  
           delay(500);
           #endif DEBUG
         */ 
           rf12_sendStart(0, payload, sizeof payload);
           //rf12_sendWait(2);  
      }
    break;
    
    case SLEEP:
    break;
  }
  
// end main loop
} 


boolean matchPayload()
{
  /*
      #ifdef DEBUG
      Serial.println("matching payload");
      delay(250);
      #endif DEBUG
    */    
  // check message length to see if it is the same as the payload we're looking for
  if ( rf12_len != sizeof payload)
    return false;
    
  memcpy(&inPayload, (byte*) rf12_data, sizeof inPayload);
  
  // test each character in the message for a match  
  for (byte i = 0; i < PayloadLength; ++i)
    if (payload[i] != char(inPayload[i]))
       return false;
 
 #ifdef DEBUG
 Serial.println("MATCHED");
 delay(250);
 #endif DEBUG
        
 
  // otherwise, matched it 
  return true;
}

